*-----------------------------------------------------------------------
      subroutine calc_shell_thick(thick)
*-----------------------------------------------------------------------
* thick: Resultant shell thickness value.
*
* This subroutine calculates the shell thickness based on a trajectory
* on the dominium specified by the user by a line segment (with global
* variables x1st, y1st, x2st, y2st) and a user specified fraction of 
* the solid temperature stored in the tsol global variable.
*
* Author: Rodrigo A. Marin
* Date  : March 26 1993
*-----------------------------------------------------------------------

      use posttypes
      use glob_vars

      include 'pubfunc.h'

      double precision thick

      double precision tsh, tant, tcur
      double precision frac_dist, dist
      double precision m, a, b, c
      double precision xant, yant, xcur, ycur
      integer i
      integer j
      integer nedgeinter
      integer inc
      
      logical quit

      type (intersection) intersecs(MAXEL+1)

      call minmax(x,nnodes,xmin,xmax)
      call minmax(y,nnodes,ymin,ymax)

      call calc_intersections(node, nel, x, y, nnodes, tlinest, 
     +                        x1st, y1st, x2st, y2st, ast, bst, cst,
     +                        intersecs, nedgeinter)

      call sort_intersecs(intersecs, nedgeinter)

      tsh = tsol + (1.0 - frac_tsol) * (tliq - tsol)
      thick = 0.0

      call interp_temp(intersecs, nedgeinter, 1, 1, tfirst)
      call interp_temp(intersecs, ndegeinter, nedgeinter,
     +                 intersecs(nedgeinter)%numint, tlast)

      quit = .FALSE.

      if ((tsh .le. tfirst) .and. (tsh .le. tlast)) then
          quit = .TRUE.
      else
          if (tfirst .lt. tlast) then
              i = 1
              tant = tfirst
              inc = 1
              call interp_dist(x1st, y1st, x2st, y2st, intersecs, 
     +                         nedgeinter, 1, 1, xant, yant)
          else
              i = nedgeinter
              tant = tlast
              inc = -1
              call interp_dist(x1st, y1st, x2st, y2st, intersecs, 
     +                         nedgeinter, nedgeinter, 
     +                         intersecs(nedgeinter)%numint,
     +                         xant, yant)
          end if
      end if

      j = 2
      
      do while (.not. quit)
         do while ((.not. quit) .and. (j .le. intersecs(i)%numint))
            call interp_temp(intersecs, nedgeinter, i, j, tcur)
            call interp_dist(x1st, y1st, x2st, y2st, intersecs, 
     +                       nedgeinter, i, j, xcur, ycur)

            dist = dsqrt((xcur - xant)**2.0 + (ycur - yant)**2.0)

            if ((tsh .ge. tant) .and. (tsh .le. tcur)) then
                frac_dist = (tsh - tant) / (tcur - tant)

                thick = thick + frac_dist * dist
                quit = .TRUE.
            else
                thick = thick + dist
                xant = xcur
                yant = ycur
                tant = tcur
            end if
            j = j + 1
         end do
         
         if (.not. quit) then
             i = i + inc
             quit = (i .lt. 1) .or. (i .gt. nedgeinter)
             j = 1
         end if
      end do

      return
      end

*-----------------------------------------------------------------------
      subroutine interp_dist(x1, y1, x2, y2, intersecs, nedgeinter,
     +                       edgeinter, interindx, xpt, ypt)
*-----------------------------------------------------------------------
* x1, y1, x2, y2: End points of a line segment.
* intersecs     : Array of intersections between this line segment and
*                the edges of the domain.
* nedgeinter    : Number of intersections in the intersec array.
* edgeinter     : Index of the desired intersection in the intersec
*                 array.
* interindx     : Index of the point of intersection (1 or 2) in the
*                 edge intersection specified by intersecs[edgeinter].
* xpt, ypt      : Coordinates of the intersection point specified by the
*                 above parameters, using a linear interpolation.
*
* This subroutine interpolates the coordinates of a point on the
* given line segment, specified by the indicated point intersection 
* on the indicated line segment - edge intersection.
*
* Author: Rodrigo A. Marin
* Date  : March 26 1993
*-----------------------------------------------------------------------

      use posttypes

      double precision x1, y1, x2, y2
     
      integer nedgeinter
      type (intersection) intersecs(nedgeinter)
      integer edgeinter, interindx
     
      double precision xpt, ypt

      double precision alpha

      alpha = intersecs(edgeinter)%t2(interindx)

      xpt = alpha * x2 + (1.0 - alpha) * x1

      ypt = alpha * y2 + (1.0 - alpha) * y1

      return
      end

*-----------------------------------------------------------------------
      subroutine interp_temp(intersecs, nedgeinter, edgeinter, 
     +                       interindx, temperature)
*-----------------------------------------------------------------------
* intersecs : Line segment - domain edges intersections array.
* nedgeinter: Number of intersections in the previous array.
* edgeinter : Index of the desired line segment - edge intersection.
* interindx : Index (1 or 2) of the point intersection on the given
*             line segment - edge intersection.
* temperature: Interpolated temperature.
*
* This subroutine performs a linear interpolation of the nodal temperature
* at the intersection specified intersection point.
*
* Author: Rodrigo A. Marin
* Date  : March 26 1993
*-----------------------------------------------------------------------

      use posttypes
      use glob_vars

      integer nedgeinter
      type (intersection) intersecs(nedgeinter)
     
      integer edgeinter
      integer interindx

      double precision temperature

      integer n1, n2
      double precision beta
      double precision v1, v2


      n1 = node(intersecs(edgeinter)%edgenum, 
     +          intersecs(edgeinter)%elnum)
      n2 = node(mod(intersecs(edgeinter)%edgenum, 3) + 1, 
     +          intersecs(edgeinter)%elnum)

      beta = intersecs(edgeinter)%t1(interindx)

      v1 = varvals(1, n1)
      v2 = varvals(1, n2)

      temperature = beta * v2 + (1.0 - beta) * v1

      return
      end

*-----------------------------------------------------------------------
      subroutine read_shell_thick_input_data
*-----------------------------------------------------------------------
* This subroutine reads from the user the shell thickness parameters:
* linear trajectory of calculation and fraction of solid temperature.
*
* Author: Rodrigo A. Marin.
* Date  : March 26 1993
*-----------------------------------------------------------------------

      use posttypes
      use glob_vars

      include 'pubfunc.h'

      double precision dxst, dyst
      double precision new_frac_tsol
      double precision values(4)

      double precision x1d, y1d, x2d, y2d

      logical quit1

      integer option

      character prompt*(MAXPROMPTLENGTH)

      call setplotconfig(mtitle, 'n', 'n', 'n', 'y', 'n', 'n',
     +                   intlab, disp_border_plot)
      call asklab(ptitle,nodplot,nodnum,elplot,boundplt,
     +            contlabl, intlab, iunit,iout,iecho)
c-----------------------------------------------------------------------
c     Initialize terminal
      call hf_open_segment('?Picture/plot')

c----------------Plot nodes and nodes numbers if requested--------------
      if (nodplot .eq. 'y') call plotn(x,y,xwmin, ywmin,
     +                                 xwmax, ywmax, nn,nodnum)
c
c----------------Draw elements if requested-----------------------------
      if (elplot .eq. 'y')
     +    call eplot(x,y,nn,nel,node,intrash)
c
c----------------Draw boundary if requested-----------------------------
      if (boundplt .eq. 'y')
     +    call border(x,y,nn,nbe,nbnc,0)

      call update_display()

      call readmenu(12, option, iunit, iout)

      if (option .eq. 1) then
          write(iout, *)
          write(iout, *)'Please use the graphics window'
          write(iout, *)
          call clear_messages()
          call window_definition(3)
          call message(1)
      else if (option .eq. 2) then
               write(iout, *)
               write(iout, 100) xwmin, ywmin, xwmax, ywmax
               write(iout, *)
               call define_new_window(xmin, ymin, xmax, ymax,
     +                                xwmin, ywmin, xwmax, ywmax,
     +                                iunit, iout)
               call setup_window(xwmin,ywmin,xwmax,ywmax)
      end if

      call show_cut_plane(x1st, y1st, x2st, y2st)

      call update_display()

      quit1 = .FALSE.

      do while (.not. quit1)
         write(iout, *)
         write(iout, 1150) x1st, y1st, x2st, y2st
         write(iout, *)

         call readmenu(13, option, iunit, iout)
         
         if (option .eq. 1) then
             write(iout, *)'Please use the graphics window'
             write(iout, *)
             call clear_messages()
             call hf_flush_contents('?Picture/plot/cutplane',
     +                              'lines')
             call read_line('?Picture/plot/cutplane', values(1),
     +                      values(2), values(3), values(4))
             call message(1)
             nval = 4
         else if (option .eq. 2) then
              values(1) = x1st
              values(2) = y1st
              values(3) = x2st
              values(4) = y2st

              write(iout, *)
              write(iout, 1200) xmin, ymin, xmax, ymax
              write(iout, *)
              write(prompt, 1100) x1st, y1st, x2st, y2st

              write(iout, *) 
     +        'The end points are specified as  x1,y1,x2,y2'
              write(iout, *)
              call readndouble(prompt, strlen(prompt), 4, .TRUE.,
     +                         nval, values, iunit, iout)
         else if (option .eq. 3) then
              values(1) = x1st
              values(2) = y1st
              values(3) = x2st
              values(4) = y2st
              nval = 0
         end if

         if ((nval .eq. 4) .or. (nval .eq. 0)) then
             if ((dabs(values(3) - values(1)) .le. ZERO) .and.
     +           (dabs(values(4) - values(2)) .le. ZERO)) then
                 call reperror(12, iout)
             else
                 if ((dabs(x1st - values(1)) .gt. ZERO) .or.
     +               (dabs(y1st - values(2)) .gt. ZERO) .or.
     +               (dabs(x2st - values(3)) .gt. ZERO) .or.
     +               (dabs(y2st - values(4)) .gt. ZERO)) then
                     x1st = values(1)
                     y1st = values(2)
                     x2st = values(3)
                     y2st = values(4)
                  end if

                  call classify_line(x1st, y1st, x2st, y2st, tlinest, 
     +                               mst, ast, bst, cst)

                  quit1 = .TRUE.
             end if
         else
             call reperror(20, iout)
         end if
      end do

      dxst = x2st - x1st
      dyst = y2st - y1st

      if (tlinest .eq. TLINE_LT45) then
          x1d = xwmin
          x2d = xwmax

          y1d = mst * x1d + cst / dxst
          y2d = mst * x2d + cst / dxst
      else
          y1d = ywmin
          y2d = ywmax

          x1d = mst * y1d - cst / dyst
          x2d = mst * y2d - cst / dyst
      end if

      call show_cut_plane(x1d, y1d, x2d, y2d)

      call hf_close_segment()
      call update_display()

      quit1 = .FALSE.

      do while (.not. quit1)
         write(prompt, 1300) frac_tsol

         new_frac_tsol = readdouble(prompt, strlen(prompt), .TRUE.,
     +                              frac_tsol, iunit, iout)

         if ((new_frac_tsol .ge. 0.0) .and. (new_frac_tsol .le. 1.0)) 
     +   then
            quit1 = .TRUE.
            frac_tsol = new_frac_tsol
         else
             call reperror(22, iout)
         end if
      end do

      write(iout, *)

      return
100   format('The current window is (xmin= ', f10.5, ', ymin= ', f10.5,
     +       ') - (xmax= ', f10.5, ', ymax= ', f10.5, ')')
1100  format('End points of trajectory [',
     +       F10.5, ', ', F10.5, ', ', F10.5, ', ', F10.5, ']: ')
1150  format('The current trajectory is (', F10.5, ', ', F10.5,
     +       ') - (', F10.5, ', ', F10.5, ')')
1200  format('The minimum and maximum coordinates are ',
     +       '(', F10.6, ', ', F10.6, ') - (', F10.6, ', ', F10.6, ')')
1300  format('Please enter fraction of solid temperature ',
     +       '(between 0.0 and 1.0) [', F8.3, ']: ')
      end
